window.Gleam.initSearch([{"doc":"lamb","title":"lamb","content":"# lamb\n\n[![Package Version](https://img.shields.io/hexpm/v/lamb)](https://hex.pm/packages/lamb)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lamb/)\n\nA gleam library for operating and querying ETS tables.\n\n```gleam\nimport gleam/list\nimport lamb.{Set, Private}\nimport lamb/table.{Config, Private, Set}\nimport lamb/record\nimport lamb/query.{var, ignore, atom}\n\ntype User {\n  User(name: String, age: Int, bio: String)\n}\n\npub fn main() {\n  // Create a table and insert 5 records.\n  let assert Ok(table) =\n    Config(name: &quot;users&quot;, access: Private, kind: Set, registered: False)\n    |&gt; table.create()\n\n  lamb.insert(table, &quot;00&quot;, User(&quot;Raúl&quot;, age: 35, bio: &quot;While at friends gatherings, plays yugioh.&quot;))\n  lamb.insert(table, &quot;01&quot;, User(&quot;César&quot;, age: 33, bio: &quot;While outdoors, likes bird watching.&quot;))\n  lamb.insert(table, &quot;02&quot;, User(&quot;Carlos&quot;, age: 30, bio: &quot;Always craving for coffee.&quot;))\n  lamb.insert(table, &quot;10&quot;, User(&quot;Adrián&quot;, age: 26, bio: &quot;Simply exists.&quot;))\n\n  // This query syntax builds a matchspec to make queries on the table.\n  let query =\n    q.new()\n    |&gt; q.bind(#(var(0), var(1)))\n\n  // Retrieve all rows but only return the record\n  let query_records = query |&gt; q.map(var(1))\n  let _records = lamb.all(table, query_records)\n\n  // Retrieve all rows but only return the index\n  let query_indexes = query |&gt; q.map(var(0))\n  let _indexes = lamb.all(table, query_indexes)\n\n  // Retrieve all records in batches of 2.\n  let assert Records([_, _] as a, step) = lamb.partial(table, by: 2, where: q.new())\n  let assert Records([_, _] as b, step) = lamb.continue(step)\n  let assert End([User(_, _, _, _)] as c) = lamb.continue(step)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/lamb&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n---\n\nWork in progress notes.\n\n# Table API\n\nCurrently viewing the table creating in terms of protection levels:\n  - `Private`, private table, not named.\n  - `Protected`, protected table, named.\n  - `Public`, public table, named.\n\nIf we would like more precision than this, maybe having an `options` helper with default options would\nhelp to tinker with tables:\n\n```gleam\npub fn options() -&gt; Options {\n  https://www.erlang.org/doc/apps/stdlib/ets.html#info/1\n  todo\n}\n```\n\nStill need to figure out what is going to be the API for differentiating between `set` and `bag` tables.\n\n# Query API\n\nMatchspecs are composed by a Tuple of arity 3 called a `MatchFunction`:\n\n- A `Head` that contains the shape of the data we want to match to, as well as variable declarations.\n- A list of `Condition` expressions that can help filter data through predicates.\n- A `Body` that declares the shape and variables we&#39;d like to output from the `MatchFunction`.\n\n```erlang\n{Head, [Condition], [Body]}\n```\n\nOperating on these 3 pieces may be tackled in different ways.\n\n## Alternative 1\n\nHave an API that composes matchspecs together with a builder pattern:\n\n```gleam\nlet query =\n  q.new()                                  // Builds a basic default match function.\n  |&gt; q.bind(Person)                        // Modifies the head to match the constructor.\n  |&gt; q.match(field: 5, &quot;Citizen&quot;)          // Modifies the head to match the value.\n  |&gt; q.where(field: 3, op: &quot;&gt;=&quot;, than: 18) // Adds a condition expression.\n\nquery |&gt; q.select(object())                // Returns the whole object #(index, record).\nquery |&gt; q.select(index())                 // Returns just the index of record.\nquery |&gt; q.select(#(v(2), v(1)))           // Returns the 2nd and 1st variables in a tuple.\nquery |&gt; q.select(False)                   // Returns false for each matching record.\nquery |&gt; q.select(Driver)                  // Maps the variables to a constructor.\nquery |&gt; q.select(fn(last, first) {        // Alternative mapper to above.\n  Driver(last, first)\n})\n```\n\n## Alternative 2\n\nBuild a &quot;nice&quot; parser to compose complex queries:\n\n```gleam\n\nlet query = &quot;\n  from Person\n  select 2, 1\n  where &#39;Citizen&#39; = 5\n  where 3 &gt;= 18\n  select #(0, 1, 2)\n&quot;\n```\n\n## Alternative 3\n\nJust build a more straightforward mapper for matchspecs.\n\n```gleam\nlet head = #(any(), record(&quot;Person&quot;, v(1), v(2), v(3), &quot;Citizen&quot;))\nlet condition = #(&quot;&gt;=&quot;, v(3), 18)\nlet body = body(record(&quot;Driver&quot;, v(2), v(1)))\nlet query_driver_elegibility = #(head, [condition], [body])\nlamb.search(table, [query_driver_elegibility])\n```\n\n## Notes on types and validation\n\nNone of these alternatives currently provide a good way of enforcing types but are meant to fail gracefully if\nthere are errors. Querying so far is a &quot;dynamic&quot; operation.\n\nAn alternative would be to run validations at runtime with the help of a schema but that would\nbe quite a big lift for my current purposes.\n\nMaybe another way is to provide a validator at init time, but this would exclusively check the validity of the\nmatchspecs.\n\n# Checkout these ETS APIs\n\n* https://www.erlang.org/doc/apps/stdlib/ets.html#select_delete/2\n* https://www.erlang.org/doc/apps/stdlib/ets.html#fun2ms/1\n* https://www.erlang.org/doc/apps/stdlib/ets.html#test_ms/2\n* https://www.erlang.org/doc/apps/stdlib/ets.html#table/2\n* https://www.erlang.org/doc/apps/stdlib/qlc\n* https://www.erlang.org/doc/apps/stdlib/erl_parse.html#abstract/1\n","url":"index.html"},{"doc":"lamb/query","title":"Query","content":"pub type Query(index, record, binding)\n\n\nSynonyms:\nlamb/query.Query\nlamb/query Query","url":"lamb/query.html#Query"},{"doc":"lamb/query","title":"against","content":"pub fn against(\n  query: Query(a, b, c),\n  row: d,\n) -&gt; Result(e, List(String))\n\nSynonyms:\nlamb/query.against\nlamb/query against","url":"lamb/query.html#against"},{"doc":"lamb/query","title":"bind0","content":"pub fn bind0(\n  query: Query(a, b, c),\n  with constructor: fn() -&gt; b,\n) -&gt; Query(a, b, fn() -&gt; b)\n\nSynonyms:\nlamb/query.bind0\nlamb/query bind0","url":"lamb/query.html#bind0"},{"doc":"lamb/query","title":"bind1","content":"pub fn bind1(\n  query: Query(a, b, c),\n  with constructor: fn(d) -&gt; b,\n) -&gt; Query(a, b, fn(d) -&gt; b)\n\nSynonyms:\nlamb/query.bind1\nlamb/query bind1","url":"lamb/query.html#bind1"},{"doc":"lamb/query","title":"bind2","content":"pub fn bind2(\n  query: Query(a, b, c),\n  with constructor: fn(d, e) -&gt; b,\n) -&gt; Query(a, b, fn(d, e) -&gt; b)\n\nSynonyms:\nlamb/query.bind2\nlamb/query bind2","url":"lamb/query.html#bind2"},{"doc":"lamb/query","title":"bind3","content":"pub fn bind3(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f) -&gt; b)\n\nSynonyms:\nlamb/query.bind3\nlamb/query bind3","url":"lamb/query.html#bind3"},{"doc":"lamb/query","title":"bind4","content":"pub fn bind4(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g) -&gt; b)\n\nSynonyms:\nlamb/query.bind4\nlamb/query bind4","url":"lamb/query.html#bind4"},{"doc":"lamb/query","title":"bind5","content":"pub fn bind5(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g, h) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g, h) -&gt; b)\n\nSynonyms:\nlamb/query.bind5\nlamb/query bind5","url":"lamb/query.html#bind5"},{"doc":"lamb/query","title":"bind6","content":"pub fn bind6(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g, h, i) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g, h, i) -&gt; b)\n\nSynonyms:\nlamb/query.bind6\nlamb/query bind6","url":"lamb/query.html#bind6"},{"doc":"lamb/query","title":"bind7","content":"pub fn bind7(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g, h, i, j) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g, h, i, j) -&gt; b)\n\nSynonyms:\nlamb/query.bind7\nlamb/query bind7","url":"lamb/query.html#bind7"},{"doc":"lamb/query","title":"bind8","content":"pub fn bind8(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g, h, i, j, k) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g, h, i, j, k) -&gt; b)\n\nSynonyms:\nlamb/query.bind8\nlamb/query bind8","url":"lamb/query.html#bind8"},{"doc":"lamb/query","title":"bind9","content":"pub fn bind9(\n  query: Query(a, b, c),\n  with constructor: fn(d, e, f, g, h, i, j, k, l) -&gt; b,\n) -&gt; Query(a, b, fn(d, e, f, g, h, i, j, k, l) -&gt; b)\n\nSynonyms:\nlamb/query.bind9\nlamb/query bind9","url":"lamb/query.html#bind9"},{"doc":"lamb/query","title":"map0","content":"pub fn map0(\n  query: Query(a, b, fn() -&gt; b),\n  with constructor: fn(a) -&gt; c,\n) -&gt; Query(a, b, fn(a) -&gt; c)\n\nSynonyms:\nlamb/query.map0\nlamb/query map0","url":"lamb/query.html#map0"},{"doc":"lamb/query","title":"map1","content":"pub fn map1(\n  query: Query(a, b, fn(c) -&gt; b),\n  with constructor: fn(a, c) -&gt; d,\n) -&gt; Query(a, b, fn(a, c) -&gt; d)\n\nSynonyms:\nlamb/query.map1\nlamb/query map1","url":"lamb/query.html#map1"},{"doc":"lamb/query","title":"map2","content":"pub fn map2(\n  query: Query(a, b, fn(c, d) -&gt; b),\n  with constructor: fn(a, c, d) -&gt; e,\n) -&gt; Query(a, b, fn(a, c, d) -&gt; e)\n\nSynonyms:\nlamb/query.map2\nlamb/query map2","url":"lamb/query.html#map2"},{"doc":"lamb/query","title":"map3","content":"pub fn map3(\n  query: Query(a, b, fn(c, d, e) -&gt; b),\n  with constructor: fn(a, c, d, e) -&gt; f,\n) -&gt; Query(a, b, fn(a, c, d, e) -&gt; f)\n\nSynonyms:\nlamb/query.map3\nlamb/query map3","url":"lamb/query.html#map3"},{"doc":"lamb/query","title":"map4","content":"pub fn map4(\n  query: Query(a, b, fn(c, d, e, f) -&gt; b),\n  with constructor: fn(a, c, d, e, f) -&gt; g,\n) -&gt; Query(a, b, fn(a, c, d, e, f) -&gt; g)\n\nSynonyms:\nlamb/query.map4\nlamb/query map4","url":"lamb/query.html#map4"},{"doc":"lamb/query","title":"map5","content":"pub fn map5(\n  query: Query(a, b, fn(c, d, e, f, g) -&gt; b),\n  with constructor: fn(a, c, d, e, f, g) -&gt; h,\n) -&gt; Query(a, b, fn(a, c, d, e, f, g) -&gt; h)\n\nSynonyms:\nlamb/query.map5\nlamb/query map5","url":"lamb/query.html#map5"},{"doc":"lamb/query","title":"map6","content":"pub fn map6(\n  query: Query(a, b, fn(c, d, e, f, g, h) -&gt; b),\n  with constructor: fn(a, c, d, e, f, g, h) -&gt; i,\n) -&gt; Query(a, b, fn(a, c, d, e, f, g, h) -&gt; i)\n\nSynonyms:\nlamb/query.map6\nlamb/query map6","url":"lamb/query.html#map6"},{"doc":"lamb/query","title":"map7","content":"pub fn map7(\n  query: Query(a, b, fn(c, d, e, f, g, h, i) -&gt; b),\n  with constructor: fn(a, c, d, e, f, g, h, i) -&gt; j,\n) -&gt; Query(a, b, fn(a, c, d, e, f, g, h, i) -&gt; j)\n\nSynonyms:\nlamb/query.map7\nlamb/query map7","url":"lamb/query.html#map7"},{"doc":"lamb/query","title":"map8","content":"pub fn map8(\n  query: Query(a, b, fn(c, d, e, f, g, h, i, j) -&gt; b),\n  with constructor: fn(a, c, d, e, f, g, h, i, j) -&gt; k,\n) -&gt; Query(a, b, fn(a, c, d, e, f, g, h, i, j) -&gt; k)\n\nSynonyms:\nlamb/query.map8\nlamb/query map8","url":"lamb/query.html#map8"},{"doc":"lamb/query","title":"map9","content":"pub fn map9(\n  query: Query(a, b, fn(c, d, e, f, g, h, i, j, k) -&gt; b),\n  with constructor: fn(a, c, d, e, f, g, h, i, j, k) -&gt; l,\n) -&gt; Query(a, b, fn(a, c, d, e, f, g, h, i, j, k) -&gt; l)\n\nSynonyms:\nlamb/query.map9\nlamb/query map9","url":"lamb/query.html#map9"},{"doc":"lamb/query","title":"new","content":"pub fn new() -&gt; Query(a, b, c)\n\nSynonyms:\nlamb/query.new\nlamb/query new","url":"lamb/query.html#new"},{"doc":"lamb/query","title":"validate","content":"pub fn validate(\n  query: Query(a, b, c),\n) -&gt; Result(Query(a, b, c), List(String))\n\nSynonyms:\nlamb/query.validate\nlamb/query validate","url":"lamb/query.html#validate"},{"doc":"lamb/query","title":"var","content":"pub fn var(at position: Int) -&gt; Atom\n\nSynonyms:\nlamb/query.var\nlamb/query var","url":"lamb/query.html#var"},{"doc":"lamb/query","title":"lamb/query","content":"","url":"lamb/query.html"},{"doc":"lamb","title":"Access","content":"pub type Access {\n  Public\n  Protected\n  Private\n}\n\nPublic\n\n\nProtected\n\n\nPrivate\n\n\nSynonyms:\nlamb.Access\nlamb Access","url":"lamb.html#Access"},{"doc":"lamb","title":"Config","content":"pub type Config {\n  Config(\n    name: String,\n    access: Access,\n    kind: Kind,\n    registered: Bool,\n  )\n}\n\nConfig(\n  name: String,\n  access: Access,\n  kind: Kind,\n  registered: Bool,\n)\n\n\nSynonyms:\nlamb.Config\nlamb Config","url":"lamb.html#Config"},{"doc":"lamb","title":"Error","content":"pub type Error {\n  AlreadyRegistered(Name)\n}\n\nAlreadyRegistered(Name)\n\n\nSynonyms:\nlamb.Error\nlamb Error","url":"lamb.html#Error"},{"doc":"lamb","title":"Kind","content":"pub type Kind {\n  Set\n  Bag\n}\n\nSet\n\n\nBag\n\n\nSynonyms:\nlamb.Kind\nlamb Kind","url":"lamb.html#Kind"},{"doc":"lamb","title":"Partial","content":"pub type Partial(record) {\n  Records(List(record), Step)\n  End(List(record))\n}\n\nRecords(List(record), Step)\n\n\nEnd(List(record))\n\n\nSynonyms:\nlamb.Partial\nlamb Partial","url":"lamb.html#Partial"},{"doc":"lamb","title":"Step","content":"pub type Step\n\n\nSynonyms:\nlamb.Step\nlamb Step","url":"lamb.html#Step"},{"doc":"lamb","title":"Table","content":"pub type Table(index, record) {\n  Table(\n    reference: TableId,\n    name: Name,\n    kind: Kind,\n    access: Access,\n    registered: Bool,\n  )\n}\n\nTable(\n  reference: TableId,\n  name: Name,\n  kind: Kind,\n  access: Access,\n  registered: Bool,\n)\n\n\nSynonyms:\nlamb.Table\nlamb Table","url":"lamb.html#Table"},{"doc":"lamb","title":"all","content":"pub fn all(\n  from table: Table(a, b),\n  where query: Query(a, b, c),\n) -&gt; List(d)\n\nSynonyms:\nlamb.all\nlamb all","url":"lamb.html#all"},{"doc":"lamb","title":"batch","content":"pub fn batch(\n  from table: Table(a, b),\n  by limit: Int,\n  where query: Query(a, b, c),\n) -&gt; Partial(d)\n\nSynonyms:\nlamb.batch\nlamb batch","url":"lamb.html#batch"},{"doc":"lamb","title":"continue","content":"pub fn continue(step: Step) -&gt; Partial(a)\n\nSynonyms:\nlamb.continue\nlamb continue","url":"lamb.html#continue"},{"doc":"lamb","title":"count","content":"pub fn count(\n  from table: Table(a, b),\n  where query: Query(a, b, c),\n) -&gt; Int\n\nSynonyms:\nlamb.count\nlamb count","url":"lamb.html#count"},{"doc":"lamb","title":"create","content":"pub fn create(config: Config) -&gt; Result(Table(a, b), Error)\n\nSynonyms:\nlamb.create\nlamb create","url":"lamb.html#create"},{"doc":"lamb","title":"delete","content":"pub fn delete(table: Table(a, b)) -&gt; Nil\n\nSynonyms:\nlamb.delete\nlamb delete","url":"lamb.html#delete"},{"doc":"lamb","title":"erase","content":"pub fn erase(\n  table: Table(a, b),\n  where query: Query(a, b, c),\n) -&gt; Int\n\nSynonyms:\nlamb.erase\nlamb erase","url":"lamb.html#erase"},{"doc":"lamb","title":"from_name","content":"pub fn from_name(name: String) -&gt; Result(Reference, Nil)\n\nSynonyms:\nlamb.from_name\nlamb from_name","url":"lamb.html#from_name"},{"doc":"lamb","title":"insert","content":"pub fn insert(table: Table(a, b), index: a, record: b) -&gt; Nil\n\nSynonyms:\nlamb.insert\nlamb insert","url":"lamb.html#insert"},{"doc":"lamb","title":"is_alive","content":"pub fn is_alive(table: Table(a, b)) -&gt; Bool\n\nSynonyms:\nlamb.is_alive\nlamb is_alive","url":"lamb.html#is_alive"},{"doc":"lamb","title":"lamb","content":"","url":"lamb.html"},{"doc":"lamb/experimental","title":"lamb/experimental","content":"","url":"lamb/experimental.html"}]);